﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace ZombiesAreDerp
{
    public class Zombie
    {
        public int ground { get; set; }
        public float walkingSpeed { get; set; }
        public Texture2D zombieTexture { get; set; }
        public Vector2 zombiePosition { get; set; }
        public SpriteEffects effect { get; set; }
        public bool doNotDraw { get; set; }
        public int zombieHealth { get; set; }
        private bool remove = false;


        /// <summary>
        /// Generates a new zombie class
        /// </summary>
        /// <param name="ZombieTexture">Texture2D | Texture of the zombie.</param>
        /// <param name="ZombiePosition">Vector2 | Position of the zombie.</param>
        /// <param name="ground">Int Height of | ground at the bottom of the screenheight. Default is 50.</param>
        /// <param name="WalkingSpeed">Float | Walking speed of the zombie. Default is 0.1.</param>
        /// <param name="ZombieHeight">Int | The height of the zombie with it's texture. Default is 192 pixel.</param>
        /// <param name="HeightOfMap">Int | The height of the map, or the height of the screen.</param>
        public Zombie(Texture2D ZombieTexture, Vector2 ZombiePosition, int ground = 50, float WalkingSpeed = 0.2f, int ZombieHeight = 192, int HeightOfMap = 600)
        {
            this.zombieTexture = ZombieTexture;
            this.zombiePosition = ZombiePosition;
            this.ground = HeightOfMap - (ZombieHeight + ground);
            this.effect = SpriteEffects.None;
            this.walkingSpeed = WalkingSpeed;
            this.doNotDraw = false;
            this.zombieHealth = 100;
        }

        /// <summary>
        /// Update the zombie
        /// </summary>
        /// <param name="gt">GameTime</param>
        /// <param name="targetPositon">Vector2 | Target position of the zombie, used for killing the target.</param>
        /// <param name="stickToGround">Boolean | Must the zombie float to the target or should it stick to the ground?</param>
        public void Update(GameTime gt, Vector2 targetPositon, bool stickToGround = true)
        {
            if (!remove)
            {
                moveTowardsTarget(targetPositon, gt, stickToGround);

                if (targetPositon.X > zombiePosition.X)//sprite effects
                {
                    this.effect = SpriteEffects.None;
                }
                else
                {
                    this.effect = SpriteEffects.FlipHorizontally;
                }
            }
        }

        /// <summary>
        /// Move towards target. Is already called by the update void.
        /// </summary>
        /// <param name="targetPosition">Vector2 | TargetPosition</param>
        /// <param name="gt">GameTime</param>
        /// <param name="stickToGround">Boolean | Must the zombie stick to the ground?</param>
        public void moveTowardsTarget(Vector2 targetPosition, GameTime gt, bool stickToGround = true)
        {
            Vector2 difference = targetPosition - this.zombiePosition;
            difference.Normalize();

            this.zombiePosition += difference * (float)gt.ElapsedGameTime.TotalMilliseconds * this.walkingSpeed;
            if (stickToGround)
            {
                zombiePosition = new Vector2(this.zombiePosition.X, this.ground);
            }
        }

        /// <summary>
        /// Checks if the zombie is hit. Should be in the update code.
        /// </summary>
        /// <param name="weaknessPosition">The position of the weakness.(bullet, stone, etc.)</param>
        /// <param name="weaknessTexture">The texture of the weakness.(bullet, stone, etc)</param>
        /// <returns>Boolean value</returns>
        public bool isZombieHit(Vector2 weaknessPosition, Texture2D weaknessTexture)
        {
            //weakness midpoints
            int x = (int)weaknessPosition.X + (weaknessTexture.Width / 2);
            int y = (int)weaknessPosition.Y + (weaknessTexture.Height / 2);

            if (this.zombiePosition != null && this.zombieTexture != null)
            {
                if (x > this.zombiePosition.X && x < this.zombiePosition.X + this.zombieTexture.Width && y > this.zombiePosition.Y && y < this.zombiePosition.Y + this.zombieTexture.Height)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Checks if the target is hit by the zombie. Should be in the update code.
        /// </summary>
        /// <param name="targetPosition">The position of the target.</param>
        /// <param name="targetTexture">The texture of the target.</param>
        /// <returns>Boolean value</returns>
        public bool isTargetHit(Vector2 targetPosition, Texture2D targetTexture)
        {
            //midpoints targets
            int x = (int)targetPosition.X + (targetTexture.Width / 2);
            int y = (int)targetPosition.Y + (targetTexture.Height / 2);

            if (this.zombiePosition != null && this.zombieTexture != null)
            {
                if (x > this.zombiePosition.X && x < this.zombiePosition.X + this.zombieTexture.Width && y > this.zombiePosition.Y && y < this.zombiePosition.Y + this.zombieTexture.Height)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Respawns the Zombie.
        /// </summary>
        /// <param name="tex">Texture2D | ZombieTexture</param>
        /// <param name="pos">Vector2 | ZombiePosition</param>
        public void respawnZombie(Texture2D tex, Vector2 pos)
        {
            this.zombiePosition = pos;
            this.zombieTexture = tex;
            this.remove = false;
            this.doNotDraw = false;
        }

        /// <summary>
        /// Removes the zombie.
        /// </summary>
        public void removeZombie()/////////////////UBBGED AS HELLLLLLH!
        {
            this.zombiePosition = new Vector2(0, ground);
            this.remove = true;
            this.doNotDraw = false;//YOOOOOOOOOOOOOOOOU ARE THE CUASE OF THIS PROBLEMMMOOOOOOO!
        }

        /// <summary>
        /// draws the zombies. Completely optional void. Only needed for quick draw without other stuff.
        /// </summary>
        /// <param name="gt">GameTime</param>
        public void draw(GameTime gt, SpriteBatch spriteBatch)
        {
            Rectangle rect = new Rectangle((int)zombiePosition.X, (int)zombiePosition.Y, zombieTexture.Width, zombieTexture.Height);

            spriteBatch.Begin();
            spriteBatch.Draw(zombieTexture, rect, null, Color.White, 0.0f, Vector2.Zero, effect, 1);
            spriteBatch.End();
        }
    }
}
