using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace ZombiesAreDerp
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class mainGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont font;
        KeyboardState oldstate;
        KeyboardState newstate;
        int screenWidth = 1000;
        int screenHeight = 600;
        int ground;
        Camera camera;

        //==========BADASS============\\
        public Rectangle badassRectangle;
        public Texture2D badassTexture;
        public Vector2 badassPosition;
        public bool badassJump = false;
        public float badassVelocityUp = 21f;
        public SpriteEffects badassMirror = SpriteEffects.None;
        public int health = 1000;
        public int maxX1 = 500;
        public int maxX2 = 3529;

        //=========BULLETS============\\
        Texture2D bulletTex;
        Vector2 bulletPos;
        bool bulletShoot = false;
        bulletDirection bulletDir = bulletDirection.left;
        bool bulletDone = false;
        bool bulletDoNotSpam = false;
        float reloadTime = 0.0f;
        enum bulletDirection
        {
            left,
            right
        }

        //========BACKGROUND==========\\
        Texture2D backTexture;
        Vector2 backPosition;
        
        //=========ZOMBEHS!===========\\
        static int numberOfZombies = 100;
        int distanceBetweenZombies = 58;//in pixel
        int wave = 1;
        Zombie[] zombie = new Zombie[numberOfZombies];
        Texture2D[] zombieTexture = new Texture2D[numberOfZombies];
        Vector2[] zombiePosition = new Vector2[numberOfZombies];
        Rectangle[] zombieRect = new Rectangle[numberOfZombies];
        

        //div
        public enum GameState
        {
            Playing,
            MainMenu,
            GameOver
        }
        public GameState gameState = GameState.Playing;
        public bool debugMode = true;
        Color debugTextColor = Color.Violet;
        Texture2D healtbarTex;
        Vector2 healtbarPos;
        public int score = 0;


        public mainGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            //de camera setten
            camera = new Camera(GraphicsDevice.Viewport);
            
            //de screenWidth en Height setten
            graphics.PreferredBackBufferWidth = screenWidth;
            graphics.PreferredBackBufferHeight = screenHeight;
            graphics.ApplyChanges();
            ground = screenHeight - 256 - 50;

            //de posities setten
            backPosition = new Vector2(0, 0);
            badassPosition = new Vector2((4096 / 2) - 64, ground);

            for (int i = 0; i < numberOfZombies; i++)
            {
                zombiePosition[i] = new Vector2(0 + (distanceBetweenZombies * i), ground - 64);
            }

            bulletPos = new Vector2(badassPosition.X, badassPosition.Y);
            healtbarPos = new Vector2(badassPosition.X - 450, 20);
            
            //diversen
            oldstate = Keyboard.GetState();
            base.Initialize();
        }
        protected override void LoadContent()
        {
            backTexture = Content.Load<Texture2D>("backGround");
            badassTexture = Content.Load<Texture2D>("badAss");
            for (int i = 0; i < numberOfZombies; i++)
            {
                zombieTexture[i] = Content.Load<Texture2D>("zombie");
                zombie[i] = new Zombie(zombieTexture[i], zombiePosition[i]);
            }
            bulletTex = CreateEmptySprite(16, 6, Color.Yellow);
            healtbarTex = CreateEmptySprite(health / 10, 20, Color.Red);

            font = Content.Load<SpriteFont>("mainFont");
            spriteBatch = new SpriteBatch(GraphicsDevice);
            numberOfZombies = 10;
        }
        protected override void UnloadContent()
        {
            // something what you don't need.
        }

        protected override void Update(GameTime gameTime)
        {
            if (gameState == GameState.Playing)
            {
                if (debugMode) { updateDebug(gameTime); }
                updateBadass(gameTime);
                updateZombies(gameTime);
                updateInput();

                badassRectangle = new Rectangle((int)badassPosition.X, (int)badassPosition.Y, badassTexture.Width, badassTexture.Height);
                camera.Update(gameTime, badassPosition, badassRectangle, screenWidth, screenHeight);
            }
            else
            {
                if (gameState == GameState.GameOver)
                {
                    updateInput(true);   
                }
            }
            base.Update(gameTime);
        }
        void updateInput(bool gameOver = false)
        {
            newstate = Keyboard.GetState();

            //escape(afsluiten)
            if (newstate.IsKeyDown(Keys.Escape) && !oldstate.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }
            if (!gameOver)
            {
                //f12(debug toggle)
                if (newstate.IsKeyDown(Keys.F12) && !oldstate.IsKeyDown(Keys.F12))
                {
                    if (!debugMode)
                    {
                        debugMode = true;
                    }
                    else
                    {
                        debugMode = false;
                    }
                }
                //pijltje links(links)
                if (newstate.IsKeyDown(Keys.Left) && !oldstate.IsKeyDown(Keys.Left) && badassPosition.X > maxX1)
                {
                    badassPosition.X -= 5;
                    badassMirror = SpriteEffects.FlipHorizontally;
                }
                //pijltje recths(rechts)
                if (newstate.IsKeyDown(Keys.Right) && !oldstate.IsKeyDown(Keys.Right) && badassPosition.X < maxX2)
                {
                    badassPosition.X += 5;
                    badassMirror = SpriteEffects.None;
                }
                //pijltje boven(badassJump)
                if (newstate.IsKeyDown(Keys.Up) && !oldstate.IsKeyDown(Keys.Up))
                {
                    badassJump = true;
                }
                //spatiebalk(shoot that zombie!)
                if (newstate.IsKeyDown(Keys.Space) && !oldstate.IsKeyDown(Keys.Space) && !bulletDoNotSpam)
                {
                    bulletDoNotSpam = true;
                    bulletShoot = true;
                }
            }
        }
        void updateBadass(GameTime gt)
        {
            if (badassJump)
            {
                int vel = (int)gt.ElapsedGameTime.TotalMilliseconds;
                badassVelocityUp -= vel / 15;
                badassPosition.Y -= badassVelocityUp;

                if (badassPosition.Y > ground)
                {
                    badassVelocityUp = 21f;
                    badassPosition.Y = ground;
                    badassJump = false;
                }
            }

            for (int i = 0; i < numberOfZombies; i++)
            {
                if (zombie[i].isTargetHit(badassPosition, badassTexture))
                {
                    health--;
                }
            }

            switch (bulletShoot)
            {
                case true:
                    {
                        if (!bulletDone)
                        {
                            if (badassMirror == SpriteEffects.FlipHorizontally)
                            {
                                bulletDir = bulletDirection.left;
                            }
                            else
                            {
                                bulletDir = bulletDirection.right;
                            }
                            bulletDone = true;
                        }
                        if (bulletDir == bulletDirection.left)
                        {
                            bulletPos.X -= 10;
                        }
                        else
                        {
                            bulletPos.X += 10;
                        }
                        for (int i = 0; i < numberOfZombies; i++)
                        {
                            if (zombie[i].isZombieHit(bulletPos, bulletTex))
                            {
                                zombie[i].zombieHealth -= 34;//temp
                                if (zombie[i].zombieHealth < 1)
                                {
                                    zombie[i].zombieHealth = 100;
                                    zombie[i].removeZombie();
                                    numberOfZombies--;
                                    score += 10;
                                }
                                bulletShoot = false;
                                bulletDone = false;
                            }
                            if (bulletPos.X < 0 || bulletPos.X > 4096)
                            {
                                bulletShoot = false;
                                bulletDone = false;
                            }
                        }
                        break;
                    }
                case false:
                    {
                        reloadTime += (float)gt.ElapsedGameTime.TotalMilliseconds;
                        if (reloadTime >= 500.0f)
                        {
                            bulletDoNotSpam = false;
                            reloadTime = 0.0f;
                        }
                        bulletPos.Y = badassPosition.Y + 133;
                        if (badassMirror == SpriteEffects.None)
                        {
                            bulletPos.X = badassPosition.X + badassTexture.Width - 16;
                        }
                        else
                        {
                            bulletPos.X = badassPosition.X;
                        }
                        break;
                    }
                default:
                    {
                        break;
                    }
            }

            if (health <= 0)
            {
                gameState = GameState.GameOver;
            }
            if (health >= 0)
            {
                int i = (health / 11) * 10;
                if (i > 0)
                {
                    healtbarTex = CreateEmptySprite(i, 20, Color.Red);
                }
            }
            healtbarPos = new Vector2(badassPosition.X - 400, 20);
        }
        void updateDebug(GameTime gt)
        {

        }
        void updateZombies(GameTime gt)
        {
            if (numberOfZombies > 0)
            {
                for (int i = 0; i < numberOfZombies; i++)
                {
                    zombie[i].walkingSpeed = (i * (((float)wave / 1000) + 0.008f)) + 0.05f;//snelheid van de zombies
                    zombie[i].Update(gt, badassPosition, true);
                    if (zombie[i].zombiePosition != null && zombie[i].zombieTexture != null)
                    {
                        zombieRect[i] = new Rectangle((int)zombie[i].zombiePosition.X, (int)zombie[i].zombiePosition.Y, zombie[i].zombieTexture.Width, zombie[i].zombieTexture.Height);
                    }
                    //isZombieHit is in de bulletShoot code!!!!
                }
            }
            else
            {
                addWave();
                for (int i = 0; i < numberOfZombies; i++)
                {
                    zombiePosition[i] = new Vector2(0 + (distanceBetweenZombies * i), ground - 64);
                    zombie[i].respawnZombie(zombieTexture[i], zombiePosition[i]);
                }
            }
        }
        void addWave(int begin = 10, int increase = 5)
        {
            numberOfZombies = begin + increase;
            wave++;
        }

        public Texture2D CreateEmptySprite(int width, int height, Color color)
        {
            Texture2D tex = new Texture2D(this.GraphicsDevice, width, height, false, SurfaceFormat.Color);
            Color[] clr = new Color[width * height];
            for (int i = 0; i < clr.Length; i++)
            {
                clr[i] = color;
            }
            tex.SetData(clr);
            return tex;
        }

        protected override void Draw(GameTime gameTime)
        {
            if (gameState == GameState.Playing)
            {
                int FPS = 1000 / (int)gameTime.ElapsedGameTime.TotalMilliseconds;
                int textPosX = ((int)badassPosition.X - (badassRectangle.Width / 2)) + screenWidth / 2;
                GraphicsDevice.Clear(Color.Violet);

                spriteBatch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend, null, null, null, null, camera.transform);
                spriteBatch.Draw(backTexture, backPosition, Color.White);

                for (int i = 0; i < numberOfZombies; i++)
                {
                    if (!zombie[i].doNotDraw)
                    {
                        spriteBatch.Draw(zombie[i].zombieTexture, zombieRect[i], null, Color.White, 0.0f, Vector2.Zero, zombie[i].effect, 3); //derp zombie!
                    }
                }

                spriteBatch.Draw(bulletTex, bulletPos, Color.White);//daarna de bullet!
                spriteBatch.Draw(badassTexture, badassRectangle, null, Color.White, 0.0f, Vector2.Zero, badassMirror, 2); //badass voorop!
                spriteBatch.Draw(healtbarTex, healtbarPos, Color.White);
                spriteBatch.DrawString(font, health.ToString(), new Vector2(badassPosition.X, 20), Color.White);

                if (debugMode) // alleen als debug mode aan is
                {
                    spriteBatch.DrawString(font, "X: " + badassPosition.X, new Vector2(textPosX, 40), debugTextColor);
                    spriteBatch.DrawString(font, "Y: " + badassPosition.Y, new Vector2(textPosX, 60), debugTextColor);
                    spriteBatch.DrawString(font, "FPS: " + FPS, new Vector2(textPosX, 80), debugTextColor);
                    spriteBatch.DrawString(font, "Zombies: " + numberOfZombies, new Vector2(textPosX, 100), debugTextColor);
                }
                spriteBatch.End();
            }
            else
            {
                if (gameState == GameState.GameOver)
                {
                    spriteBatch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend, null, null, null, null, camera.transform);
                    spriteBatch.DrawString(font, "GAME OVER", new Vector2(badassPosition.X - 60, (screenHeight / 2) - 20), Color.White, 0.0f, Vector2.Zero, 2.5f, SpriteEffects.None, 0);
                    spriteBatch.End();
                }
            }

            base.Draw(gameTime);
        }
    }
}
